
##############################################
package aims2image;
##############################################

use strict;
use Digest::MD5;
use IO::File;
use File::Path;
use File::Copy;
use File::Basename;
use File::stat;

1;

##############################################
sub new
##############################################
{
   my($class, $dbh, $user, $conf) = @_;
   my($self) = bless{
      _DB => $dbh,
      _USER => $user,
      _CONF => $conf,
   }, $class;
   
   return $self;
}

##############################################
sub add
##############################################
{
   my($self, $image) = @_;
   
   # Add new pxeboot media to aims2.
   
   # Collect the physical files (and metadata)
   my %files = ( VMLINUZ => [], INITRD => [] );
   
   $files{'VMLINUZ'} = $self->collect_file( $image->{VMLINUZ} );
   if( defined($image->{INITRD}) ){
      $files{'INITRD'} = $self->collect_file( $image->{INITRD} );
   }
   
   my %record = ();
   
   $record{NAME} = $image->{NAME};
   $record{OWNER} = $self->{_USER};
   $record{ARCH} = $image->{ARCH};
   $record{DESC} = $image->{DESC};
   $record{KOPTS} = $image->{KOPTS};
      
   $record{VMLINUZ} = $image->{VMLINUZ};
   $record{VMLINUZ_FILE} = $files{'VMLINUZ'}{BLOB};
   $record{VMLINUZ_SUM} = $files{'VMLINUZ'}{SUM};
   $record{VMLINUZ_SIZE} = $files{'VMLINUZ'}{SIZE};

   $record{INITRD} = undef;
   $record{INITRD_FILE} = undef;
   $record{INITRD_SUM} = undef;
   $record{INITRD_SIZE} = undef;
      
   if( defined($image->{INITRD}) ){
      $record{INITRD} = $image->{INITRD};
      $record{INITRD_FILE} = $files{'INITRD'}{BLOB};
      $record{INITRD_SUM} = $files{'INITRD'}{SUM};
      $record{INITRD_SIZE} = $files{'INITRD'}{SIZE};
   }
   
   $self->insert_pxeboot_record(%record);
   
   $self;
}

##############################################
sub insert_pxeboot_record
##############################################
{
   my($self, %record) = @_;
   
   my $dbh = $self->{_DB};
   
   # FIXME: Better bind behaviour. 
         
   # Prepare to talk to the database.
   my @fields = qw(name owner arch description kopts vmlinuz_source vmlinuz_file vmlinuz_sum vmlinuz_size initrd_source initrd_file initrd_sum initrd_size uploaded synced synced1 synced2 synced3 synced4 synced5 synced6);#keys %record;
   my $fields = join(",",@fields);
      
   my $sql = "INSERT INTO pxeboot ($fields) VALUES (?,?,?,?,?,?,?,?,?,?,?,?,?,CURRENT_TIMESTAMP,0,'N','N','N','N','N','N')";
   #die $sql;
   # Create a new record and upload the BLOBS.
   eval {
   
      # Incase we blow our Quota to hell, so prevent AutoCommits.
      $dbh->{AutoCommit} = 0;
      
      my $prepare = $dbh->prepare($sql);
   
         $prepare->bind_param(1, $record{NAME} );
         $prepare->bind_param(2, $record{OWNER} );
         $prepare->bind_param(3, $record{ARCH} );
         $prepare->bind_param(4, $record{DESC} );
         $prepare->bind_param(5, $record{KOPTS} );
         
         $prepare->bind_param(6, $record{VMLINUZ} );
         $prepare->bind_param(7, $record{VMLINUZ_FILE}, { ora_type=>113, ora_field=>'VMLINUZ_FILE' } );
         $prepare->bind_param(8, $record{VMLINUZ_SUM});
         $prepare->bind_param(9, $record{VMLINUZ_SIZE});
         
         $prepare->bind_param(10, $record{INITRD} );
         $prepare->bind_param(11, $record{INITRD_FILE}, { ora_type=>113, ora_field=>'INITRD_FILE' } );
         $prepare->bind_param(12, $record{INITRD_SUM});
         $prepare->bind_param(13, $record{INITRD_SIZE});
         
         $prepare->execute();
	 $dbh->commit;
   };
   if($@){
   
      
      # Image already exists.
      if($@ =~ /AIMS2.BOOT_PK/){
         die "Error: $record{NAME} already exists.\n";
      }
      
      # We just blew our quota :(
      if($@ =~ /quota exceeded/){
         $dbh->rollback;
         die "Error: Database quota exceeded. Contact Linux.Support\@CERN.CH\n";
      }
      
      # Something unexpected.
      if($@ =~ $dbh->err){
         die "Database Error: Failed to create pxeboot record $record{NAME}.\n".$dbh->errstr;
      }
      
      # invalid identifier = check your fields...
      
   }
   
   $self;
}

##############################################
sub remove
##############################################
{
   my($self, $name) = @_;
   
   # Remove an image from aims2.
   
   unless($name){
      die "Error: Cannot remove. Name is undefined.\n";
   }
   
   $name =~ tr/[a-z]/[A-Z]/;
   $self->remove_pxeboot_record($name);
   
   $self;
}

##############################################
sub remove_pxeboot_record
##############################################
{
   my($self, $name) = @_;
   
   # Remove pxeboot record from the database.
   
   my $dbh = $self->{_DB};
   
   eval {
      my $prepare = $dbh->prepare("DELETE FROM pxeboot WHERE name = ?");
         $prepare->execute($name);
   };   
   if($@){      
      if($@ =~ $dbh->err){
         die "Database Error: Failed to remove $name.\n";
      }
   }
   
   $self;
}

##############################################
sub get_pxeboot_by_name
##############################################
{
   my($self, $name) = @_;
   
   # Get pxeboot information matching $name.
   
   unless($name){
      die "Error: Cannot query. NAME is undefined.\n";
   }
   $name =~ tr/*/%/;
      
   my $dbh = $self->{_DB};
      
   my @fields = qw(name owner arch kopts groups description initrd_source initrd_sum initrd_size vmlinuz_source vmlinuz_sum vmlinuz_size synced synced1 synced2 synced3 synced4 synced5 synced6);
   push(@fields, "to_char(uploaded,'yyyy/mm/dd hh24:mi:ss')");
   my $fields = join(", ", @fields);
   my $sql = "SELECT $fields FROM pxeboot WHERE name LIKE ?";
   my %pxeboot = ();
   eval {
      my $prepare = $dbh->prepare($sql);
         $prepare->execute($name);
         #$prepare->bind_columns(undef, map { \$pxeboot{$_} } @fields);
      while (my ($name,@image) = $prepare->fetchrow_array){
         (
		$pxeboot{$name}{OWNER},
		$pxeboot{$name}{ARCH},
		$pxeboot{$name}{KOPTS},
		$pxeboot{$name}{GROUPS},
		$pxeboot{$name}{DESCRIPTION},
		$pxeboot{$name}{INITRD_SOURCE},
		$pxeboot{$name}{INITRD_SUM},
		$pxeboot{$name}{INITRD_SIZE},
		$pxeboot{$name}{VMLINUZ_SOURCE},
		$pxeboot{$name}{VMLINUZ_SUM},
		$pxeboot{$name}{VMLINUZ_SIZE},
		$pxeboot{$name}{SYNCED},
                $pxeboot{$name}{SYNCED1},
		$pxeboot{$name}{SYNCED2},
		$pxeboot{$name}{SYNCED3},
                $pxeboot{$name}{SYNCED4},
                $pxeboot{$name}{SYNCED5},
                $pxeboot{$name}{SYNCED6},
		$pxeboot{$name}{UPLOADED},
         ) = @image;
      }
   };
   if($@){
      if($@ =~ $dbh->err){
         die "Database Error: Cannot retrieve pxeboot information for $name. $@\n";
      }
   }

   \%pxeboot;
}

##############################################
sub collect_file
##############################################
{
   my($self, $file) = @_;

   # Collect the file and metadata.
   
   my %f;
   
   unless(-e $file){
      die "Error: Cannot stat $file. $!.\n";
   }

   unless($file =~ m,^/afs/cern.ch/,){
      die "Error: File source should be from within /afs/cern.ch/\n";
   }

   my $size = stat($file)->size;
   my $limit = $self->{_CONF}{'UPLOAD_LIMIT'} ? $self->{_CONF}{'UPLOAD_LIMIT'} : die "Error: UPLOAD_LIMIT is undefined.\n";
   if($size > $limit){
      die "Error: This file is too big. Contact Linux.Support\@CERN.CH\n";
   }
      
   # Read the data from the file.
   my ($sum, $blob) = $self->slurp_file($file);
   
   $f{SIZE} = $size;
   $f{BLOB} = $blob;
   $f{SUM} = $sum;
 
   \%f;
}

##############################################
sub slurp_file
##############################################
{
   my($self, $file) = @_;
   
   # yumyumyum, a file to slurp.

   # FIXME: if($file =~ /(random)$/){ # /\/dev\/(random|) or something.
   #   die "Error: File looks dodgy.\n";
   #}

   my $handle = new IO::File($file, "r") or die "Error: Failed to open $file: $!.\n";
   my $ident = Digest::MD5->new;
   my $blob = "";
   while (<$handle>) {
      $blob.= $_;
      $ident->add($_);
   }

   #my $sum = $ident->digest;
   my $sum = $ident->hexdigest;
   #my $sum = $ident->b64digest;
   
   ( $sum, $blob );
   
}

##############################################
sub permission
##############################################
{
   my($self, $owner) = @_;
   
   $self->SetLDAP;
   
   my $permission = 0;
   
   # User is owner?
   if( lc($self->{_USER}) eq lc($owner) ){
      $permission++;
   }
   
   # User is member of linux support
   unless($permission > 0){
      # FIXME: conf. 
      if( $self->memberof($self->{_CONF}->{'EGROUP_LINUXSUPPORT'}) ){
      #if( $self->memberof("it-dep-fio-la-members\@cern.ch") ){
         $permission++;
      }
   }

   # User is member of CMF admins egroup
   unless($permission > 0){
      if( $self->memberof($self->{_CONF}->{'EGROUP_CMFADMINS'}) ){
          $permission++;
      }
   }

   
   unless($permission > 0){
      die "Error: Cannot remove image. Permission denied.\n";
   }
   
   $self;
}

##############################################
sub upload_permission
##############################################
{
   my($self) = @_;
   
   # Check user has permission to upload.
     
   $self->SetLDAP;
   
   my $permission = 0;
   
   # User is member of linux support
   unless($permission > 0){
      # FIXME: conf. 
      if( $self->memberof($self->{_CONF}->{'EGROUP_LINUXSUPPORT'}) ){
         $permission++;
      }
   }
   
   # User is member of upload list.
   unless($permission > 0){
      # FIXME: conf. 
      if( $self->memberof($self->{_CONF}->{'EGROUP_UPLOAD'}) ){
         $permission++;
      }
   }
   
   unless($permission > 0){
      die "Error: You do not have permission to upload pxeboot media.\n";
   }
   
   $self;
}

##############################################
sub SetLDAP
##############################################
{
   my($self) = @_;
   
   # Connect to ldap (cerndc.cern.ch).
   # Set connection.
   my $ldapserver = $self->{_CONF}->{'LDAP_SERVER'} ? $self->{_CONF}->{'LDAP_SERVER'} : die "Error: LDAP_SERVER is undefined.\n";
   my $ldapuser = $self->{_CONF}->{'LDAP_USER'} ? $self->{_CONF}->{'LDAP_USER'} : die "Error: LDAP_USER is undefined.\n";
   my $ldappass = $self->{_CONF}->{'LDAP_PASS'} ? $self->{_CONF}->{'LDAP_PASS'} : die "Error: LDAP_PASS is undefined.\n";
   my $ldapbase = $self->{_CONF}->{'LDAP_BASE'} ? $self->{_CONF}->{'LDAP_BASE'} : die "Error: LDAP_BASE is undefined.\n";
               
   my $ldap = aims2ldap->new();
   $ldap->ldap_connect($ldapserver, $ldapuser, $ldappass);
   
   $self->{_LDAP} = $ldap;
   
}

##############################################
sub memberof
##############################################
{
   my($self, $group) = @_;
   
   my $member = 0;
   
   my $ldap = $self->{_LDAP};
   
   my $ldapbase = $self->{_CONF}->{'LDAP_BASE'};
   
   my $result = $ldap->ldap_search( $ldapbase, "(cn=$self->{_USER})", ('MemberOf'));
   my @result = $ldap->fetch_values( $result, ('MemberOf') );
   
   my @memberof;
   for my $e (@result){
      # FIXME: Better regex.
      my($g, $b) = split(",", $e);
      $g =~ s/^CN=//;
      push(@memberof, $g);
   }
   
   if( grep { $group eq $_ } @memberof ){
      $member = 1;
   }
   
   $member;
}
